
#pragma once

#include <AK/Optional.h>
#include <AK/StringView.h>
#include <LibWeb/CSS/PropertyID.h>

namespace Web::CSS {

enum class PseudoElement : u8 {
    After,
    Backdrop,
    Before,
    DetailsContent,
    FileSelectorButton,
    Fill,
    FirstLetter,
    FirstLine,
    Marker,
    Placeholder,
    Selection,
    Thumb,
    Track,
    ViewTransition,
    ViewTransitionGroup,
    ViewTransitionImagePair,
    ViewTransitionNew,
    ViewTransitionOld,

    KnownPseudoElementCount,

    UnknownWebKit,
};

Optional<PseudoElement> pseudo_element_from_string(StringView);
Optional<PseudoElement> aliased_pseudo_element_from_string(StringView);
StringView pseudo_element_name(PseudoElement);

bool is_has_allowed_pseudo_element(PseudoElement);
bool pseudo_element_supports_property(PseudoElement, PropertyID);

struct PseudoElementMetadata {
    enum class ParameterType {
        None,
        PTNameSelector,
    } parameter_type;
    bool is_valid_as_function;
    bool is_valid_as_identifier;
};
PseudoElementMetadata pseudo_element_metadata(PseudoElement);

enum class GeneratedPseudoElement : u8 {
    After,
    Before,

};

Optional<GeneratedPseudoElement> to_generated_pseudo_element(PseudoElement);
PseudoElement to_pseudo_element(GeneratedPseudoElement);

}
